use argh::FromArgs;

use crate::api::ApiClient;
use crate::Error;
use crate::action::ActionExec;
use crate::config::Config;
use crate::utils::torrent_to_magnet;

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "add")]
/// add a torrent to qBittorrent (magnet or torrent file)
pub struct AddAction {
    #[argh(switch, short = 'p')]
    /// pause the torrent instead of starting immediately
    paused: bool,

    #[argh(positional)]
    /// the torrent to add
    torrent: String,
}

impl ActionExec for AddAction {
    fn exec(&self, config: &Config) -> Result<(), Error> {
        let magnet = if self.torrent.starts_with("magnet:") {
            self.torrent.clone()
        } else {
            torrent_to_magnet(&self.torrent)?
        };
        let api = ApiClient::from_config(&config)?;
        api.add(&magnet, self.paused)
    }
}
